(function($) {

    var SharePointListConfig = function() {
    };

    var throbberUrl = "/download/resources/com.atlassian.confluence.extra.sharepoint/ajax-loader.gif";
    var sitesDropDown = null;
    var sitesThrobber = null;
    var sitesErrorDiv = null;
    var sitesRetryLink = null;
    var sites = null;

	SharePointListConfig.prototype.beforeParamsSet = function(selectedParams, inserting) {
        if (inserting)
            return selectedParams; // a new instance of the macro is being inserted - no need to handle backwards-compatibility of parameters.

        return SharePointConnector.parseParams(selectedParams, AJS.log);
    };

    SharePointListConfig.prototype.fields = {
        "string" : {
            "Site" : function(param, options) {
                // create a basic field for the "site" parameter, based on the 'select' template.
                var paramDiv = AJS.clone("#macro-param-select-template");
                sitesDropDown = $("select", paramDiv);

                // because we're going to asynchronously load the set of available values for this field, the user
                // is not allowed to interact with the input field until the data is retrieved.  Show a throbber to
                // entertain the user while this is going on.
                sitesDropDown.attr("disabled", "disabled");
                paramDiv.append("<img id=\"spcon-site-loader\" src=\"" + AJS.params.contextPath + throbberUrl + "\">"); // throbber; visible by default.
                sitesThrobber = $("#spcon-site-loader", paramDiv);

                // create a placeholder element to display error text if something goes wrong with the AJAX call.
                paramDiv.append("<div class='error' id='spcon-site-error'/>");
                sitesErrorDiv = $("#spcon-site-error", paramDiv);
                sitesErrorDiv.append("<div id='spcon-site-errortext' class='error'/>");
                sitesErrorDiv.append("<span id='spcon-site-retry' style='color: blue'>Retry</span>"); // HACK: use a span instead of an anchor because the macro browser attaches an event handler to all anchor elements which I haven't been able to unbind yet.
                sitesRetryLink = $("#spcon-site-retry", sitesErrorDiv);

                // the macro browser adds an event handler to all anchors which causes them to open in a new window...
                // we need to remove this event handling on the retry link.
                sitesRetryLink.click(retryGetSitesASync);
                sitesErrorDiv.css("display", "none");

                // we need to do some trickery when the value is first set, and when the value is subsequently changed,
                // so set this up.
                if (options == null)
                    options = {};
                options.setValue = setSiteValue;

                // commence the asynchronous operation to retrieve the list of sites, pronto!
                getSitesASync();

                // Add a default "empty" option (which means to use the configured "default" site.
                sitesDropDown.append($("<option/>").attr("value", ""));

                return AJS.MacroBrowser.Field(paramDiv, sitesDropDown, options);
            },
            "List" : createSharePointAutoComplete("/rest/sharepoint/1.0/search",
                    function(val) {
                        return {
                            query: val.indexOf("/") > 0 ? val.substring(val.indexOf("/") + 1) : val,
                            siteAlias: sitesDropDown.val()
                        };
                    },
                    function (e, selected) {
                        if (!selected) {
                            return;
                        }

                        // extract the information about the selected item from the jQuery data cache for the corresponding <span> element.
                        var dataObj = $.data(selected.find("span")[0], "properties");
                        $("#macro-param-View").val(dataObj.viewName);
                        $("#macro-param-List").val(dataObj.listName);

                        this.hide();
                        e.preventDefault();
                    }),
            "View" : createSharePointAutoComplete("/rest/sharepoint/1.0/search/view",
                        function (val) {
                            return {
                                query: val,
                                list: $("#macro-param-List").val(),
                                siteAlias: sitesDropDown.val()
                            };
                        },
                        function(e, selected) {
                            if (!selected) {
                                return;
                            }

                            var dataObj = $.data(selected.find("span")[0], "properties");
                            $("#macro-param-View").val(dataObj.name);

                            this.hide();
                            e.preventDefault();
                        }),
            "Folder" : createSharePointAutoComplete("/rest/sharepoint/1.0/search/folder",
                    function(val) {
                        return {
                            query: val.indexOf("/") > 0 ? val.substring(val.indexOf("/") + 1) : val,
                            list: $("#macro-param-List").val(),
                            siteAlias: sitesDropDown.val()
                        };
                    },
                    function (e, selected) {
                        if (!selected) {
                            return;
                        }

                        // extract the information about the selected item from the jQuery data cache for the corresponding <span> element.
                        var dataObj = $.data(selected.find("span")[0], "properties");
                        $("#macro-param-Folder").val(dataObj.relativePath);

                        this.hide();
                        e.preventDefault();
                    })
        }
    };

    function setSiteValue(value) {
        // before we set the value, let's ensure that the appropriate option actually exists.
        // if this set is invoked by the macro browser as it loads a pre-existing macro, *and* the
        // asynchronous call to retrieve the list of available sites hasn't returned yet, then the option won't
        // be there.  If this is the case, then we'll temporarily add it in.
        var targetOption = $("#macro-param-Site > option[value='" + value + "']");
        if (targetOption.length == 0) {
            // it doesn't exist; we need to create it.
            var option = $("<option/>");
            option.attr("value", value);
            option.attr("title", value);
            option.text(value);
            sitesDropDown.append(option);
        }

        // now set it.
        sitesDropDown.val(value);
    }

    function createSharePointAutoComplete(searchURL, makeParams, selectionHandler) {
        return function(param, options) {
            var paramDiv = AJS.clone("#macro-param-template");
            var input = $("input", paramDiv);

            paramDiv.append("<div id=\"dd-list\" class=\"aui-dd-parent autocomplete\"></div>");
            var ddDiv = $("#dd-list", paramDiv);

            var ddOptions = {
                makeParams: makeParams,
                dropdownPlacement: function(dropDown) {
                    ddDiv.append(dropDown);
                },
                ajsDropDownOptions: {
                    selectionHandler: selectionHandler
                }
            };
            input.quicksearch(searchURL, null, ddOptions);
            return AJS.MacroBrowser.Field(paramDiv, input, options);
        }
    }

    function updateSitesDropDown() {
        // what's the currently selected value?
        var currentValue = sitesDropDown.val();

        // clear that motherfucker out.
        sitesDropDown.empty();
        sitesDropDown.append($("<option/>").attr("value", ""));

        for (var i = 0; i < sites.length; i++) {
            var site = sites[i];
            var option = $("<option/>");
            option.attr("value", site.alias);
            option.attr("title", site.serverUrl);
            option.text(site.alias + " (" + site.serverUrl + ")");
            sitesDropDown.append(option);
        }

        // restore the currently selected value.
        sitesDropDown.val(currentValue);

        // AJAX complete; ensure that the throbber is hidden and the drop-down is enabled.
        sitesThrobber.css("display", "none");
        sitesDropDown.removeAttr("disabled");

        sitesDropDown.change(function() {
            // When a value is selected in the sites drop down, tell Confluence to prime the web service client cache for that site,
            // as it is now quite likely that the user is about to make use of the site.
            // If we don't prime the cache, the first attempt to do an auto-complete on the list name field will
            // probably time-out, as priming an empty cache is an time-consuming operation.
            var requestSettings = {
                async: true,
                url: AJS.params.contextPath + "/rest/sharepoint/1.0/search/primer?siteAlias=" + sitesDropDown.children("option:selected").attr("value"),
                timeout: 30000, // 30 seconds.
                success: function(data) {
                    AJS.log("SOAP Cache priming successful.");
                },
                error: function(data) {
                    AJS.log("SOAP Cache priming failed.");
                }
            };
            $.ajax(requestSettings);
        });
    }

    function retryGetSitesASync(event) {
        event.preventDefault();

        // hide the error div again.
        sitesErrorDiv.css("display", "none");

        // show the throbber.
        sitesThrobber.css("display", "inline");

        // do it!
        getSitesASync();

        return false;
    }

    function getSitesASync() {
        var requestSettings = {
            async: true,
            url: AJS.params.contextPath + "/plugins/sharepoint/configuredSites.action",
            dataType: 'json',
            timeout: 30000, // 30 seconds.
            error: function(XMLHttpRequest, textStatus, errorThrown) {

                var errorText;
                if (textStatus == "timeout") {
                    // Should be internationalised
                    errorText = "The request to retrieve configured sites from Confluence timed out after 30 seconds.";
                }
                else {
                    // Should be internationalised
                    errorText = "The request to retrieve configured sites from Confluence failed.";
                }

                // make the error visible.
                $("#spcon-site-errortext", sitesErrorDiv).text(errorText);
                sitesErrorDiv.css("display", "inline");

                // hide the throbber.
                sitesThrobber.css("display", "none");
            },
            success: function(data) {
                if (!data.length) {
                    sitesErrorDiv.append("Configured SharePoint sites result was not in the expected format. <a href='#' onclick='getSitesASync()'>Retry</a>");
                    sites = {};
                }

                sites = data;

                // was successful! Make sure the error div is hidden.
                $("#spcon-site-errortext", sitesErrorDiv).empty();
                sitesErrorDiv.css("display", "none");

                updateSitesDropDown();
            }
        };
        $.ajax(requestSettings);
    }

    AJS.MacroBrowser.Macros["sp-list"] = new SharePointListConfig();
    AJS.MacroBrowser.Macros["sp-link"] = new SharePointListConfig();
    delete AJS.MacroBrowser.Macros["sp-link"].beforeParamsSet;
})(AJS.$);